/**********************************************************************
Author:			David Szybala
Program:		time_slider_assistant.ms
Date:			11/5/09
Description:	This script is a dockable toolbar that can manipulate
                the timeline and assist with animation
**********************************************************************/

try(cui.UnRegisterDialogBar time_assistant_rollout)catch()
try (destroyDialog time_assistant_rollout) catch()
callbacks.removeScripts #animationRangeChange id:#TSA_AniRangeCallback
callbacks.removeScripts #filePostOpen id:#TSA_FileOpenCallback
unRegisterTimeCallback TSA_get_current_frame
global time_assistant_rollout


/**********************************************************************
CRITICAL: Global Functions required for callbacks
**********************************************************************/
fn TSA_get_frame_values =
(
	time_assistant_rollout.start_frame_spinner.value = animationRange.start
	time_assistant_rollout.current_frame_spinner.value = sliderTime
	time_assistant_rollout.end_frame_spinner.value = animationRange.end
	time_assistant_rollout.step_size_spinner.value = frameRate
) --end fn

fn TSA_get_current_frame =
(
	time_assistant_rollout.current_frame_spinner.value = sliderTime
) --end fn


( --put script into local scope
	local time_assistant_ini_file = getDir #plugcfg + "\\TimeSliderAssistant.ini"
	local time_assistant_rolloutPos

	/**********************************************************************
    Adds/Removes the callbacks required for this script
    **********************************************************************/
	fn set_callback_state state =
	(
		if state then
		(
			callbacks.addScript #animationRangeChange "TSA_get_frame_values()" id:#TSA_AniRangeCallback
			callbacks.addScript #filePostOpen "TSA_get_frame_values()" id:#TSA_FileOpenCallback
			registerTimeCallback TSA_get_current_frame
		) --end if
		else
		(
			callbacks.removeScripts #animationRangeChange id:#TSA_AniRangeCallback
			callbacks.removeScripts #filePostOpen id:#TSA_FileOpenCallback
			unRegisterTimeCallback TSA_get_current_frame
		) --end else
	) --end fn
	
	/**********************************************************************
    UI creation
    **********************************************************************/
	rollout time_assistant_rollout "Time Slider Assistant"
	(		
		spinner start_frame_spinner "Start:" pos:[5,2] range:[-99999,99998,0] fieldWidth:45 type:#integer scale:1
		spinner current_frame_spinner "Current:" pos:[95,2] range:[-99999,99999,0] fieldWidth:45 type:#integer scale:1
		spinner end_frame_spinner "End:" pos:[200,2] range:[-99998,99999,100] fieldWidth:45 type:#integer scale:1
        spinner step_size_spinner "Step Size:" pos:[290,2] range:[0,99999,30] fieldWidth:45 type:#integer scale:1
		button start_frame_button "Start" pos:[410,2] width:65 height:16
		button step_back_button "Step -" pos:[480,2] width:65 height:16
		button step_half_back_button "Half Step -" pos:[550,2] width:65 height:16
		button step_half_foward_button "Half Step +" pos:[620,2] width:65 height:16
		button step_foward_button "Step +" pos:[690,2] width:65 height:16
		button end_frame_button "End" pos:[760,2] width:65 height:16

		/**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
		on time_assistant_rollout open do
		(
			start_frame_spinner.value = animationRange.start
			current_frame_spinner.value = sliderTime
			end_frame_spinner.value = animationRange.end
			step_size_spinner.value = frameRate
			
			set_callback_state true
		) --end on
		
		/**********************************************************************
        UI close event handler. Cleans up after the script
        **********************************************************************/
		on time_assistant_rollout close do set_callback_state false

		/**********************************************************************
        Start spinner event handler. Adjusts the start frame
        **********************************************************************/
		on start_frame_spinner changed the_value do
		(
			if the_value < end_frame_spinner.value then
				animationRange = interval the_value animationRange.end
			else
			(
				start_frame_spinner.value = end_frame_spinner.value - 1
				animationRange = interval start_frame_spinner.value animationRange.end
			) --end else
		) --end on

		/**********************************************************************
        Current spinner event handler. Adjusts the current frame
        **********************************************************************/
		on current_frame_spinner changed the_value do
		(
			if the_value >= start_frame_spinner.value and the_value <= end_frame_spinner.value then
				sliderTime = the_value
			else
				if the_value < start_frame_spinner.value then
					sliderTime = start_frame_spinner.value
				else
					sliderTime = end_frame_spinner.value
		) --end on
		
		/**********************************************************************
        End spinner event handler. Adjusts the end frame
        **********************************************************************/
		on end_frame_spinner changed the_value do
		(
			if the_value > start_frame_spinner.value then
				animationRange = interval animationRange.start the_value
			else
			(
				end_frame_spinner.value = start_frame_spinner.value + 1
				animationRange = interval animationRange.start end_frame_spinner.value
			) --end else
		) --end on
		
		/**********************************************************************
        Start button event handler. Sets the current frame to the start frame
        **********************************************************************/
		on start_frame_button pressed do sliderTime = animationRange.start
        
        /**********************************************************************
        Step back event handler. Moves the current frame one step size back
        **********************************************************************/
        on step_back_button pressed do
		(
            if (sliderTime - step_size_spinner.value) < animationRange.start then
                sliderTime = animationRange.start
            else
                sliderTime = sliderTime - step_size_spinner.value
        ) --end on
        
        /**********************************************************************
        Step half back event handler. Moves the current frame one half step
        size back
        **********************************************************************/
        on step_half_back_button pressed do
		(
            local step_size = (step_size_spinner.value * 0.5) as integer
            
            if (sliderTime - step_size < animationRange.start) then
                sliderTime = animationRange.start
            else
                sliderTime = sliderTime - step_size
        ) --end on
        
        /**********************************************************************
        Step half forward event handler. Moves the current frame one half step
        size forward
        **********************************************************************/
        on step_half_foward_button pressed do
		(
            local step_size = (step_size_spinner.value * 0.5) as integer
            
            if (sliderTime + step_size > animationRange.end) then
                sliderTime = animationRange.end
            else
                sliderTime = sliderTime + step_size
        ) --end on
        
        /**********************************************************************
        Step forward event handler. Moves the current frame one step size forward
        **********************************************************************/
        on step_foward_button pressed do
		(
            if (sliderTime + step_size_spinner.value) > animationRange.end then
                sliderTime = animationRange.end
            else
                sliderTime = sliderTime + step_size_spinner.value
        ) --end on
		
		/**********************************************************************
        End button event handler. Sets the current frame to the end frame
        **********************************************************************/
		on end_frame_button pressed do sliderTime = animationRange.end
	) --end rollout
    
	time_assistant_rolloutPos = execute (getIniSetting time_assistant_ini_file "Dialog" "Position")
	if time_assistant_rolloutPos == OK do time_assistant_rolloutPos = [100,100]
	if time_assistant_rolloutPos.x > sysinfo.desktopsize.x - 835 do time_assistant_rolloutPos.x = sysinfo.desktopsize.x - 835
	if time_assistant_rolloutPos.y > sysinfo.desktopsize.y - 85 do time_assistant_rolloutPos.y = sysinfo.desktopsize.y - 85
	if time_assistant_rolloutPos.x < 0 do time_assistant_rolloutPos.x = 0
	if time_assistant_rolloutPos.y < 0 do time_assistant_rolloutPos.y = 0
	createDialog time_assistant_rollout 835 25 time_assistant_rolloutPos.x time_assistant_rolloutPos.y
	cui.RegisterDialogBar time_assistant_rollout style:#(#cui_dock_horz,#cui_floatable,#cui_handles)
	cui.DockDialogBar time_assistant_rollout #cui_dock_bottom
) --end local scope
